package tp03.ex02;

import java.util.Arrays;
import java.time.LocalDate;

public class ToDoList {
	private Task[] chores;

	public ToDoList() {
		this.chores = new Task[5];
	}

	public void enlarge() {
		this.chores = Arrays.copyOf(this.chores, this.chores.length + 5);
	}

	public String toString() {
		String result = "" + this.getNbTask() + " tasks: \n";
		for (int i = 0; i < this.chores.length; i++) {
			if (this.chores[i] != null) {
				result += " - " + this.chores[i] + "\n";
			}
		}
		return result;
	}

	public void addTask(Task task) {
		boolean added = false;
		for (int i = 0; i < this.chores.length; i++) {
			if (this.chores[i] == null && !added) {
				this.chores[i] = task;
				added = true;
			}
		}

		// Enlarge if full
		if (!added) {
			this.enlarge();
			this.chores[this.chores.length - 5] = task;
		}
	}

	public void removeTask(int index) {
		this.chores[index] = null;
	}

	public boolean isOverwhelmed() {
		return this.getNbTask() == this.chores.length;
	}

	public int getNbTask() {
		int nb = 0;
		for (int i = 0; i < this.chores.length; i++) {
			if (this.chores[i] != null) {
				nb++;
			}
		}
		return nb;
	}

	public void onSickLeave(int days) {
		for (int i = 0; i < this.chores.length; i++) {
			if (this.chores[i] != null) {
				this.chores[i].delay(days);
			}
		}
	}

	public void emergencySort() {
		int n = this.chores.length;
        for (int i = 1; i < n; i++) {
            Task cle = this.chores[i];
            int j = i - 1;

            // Déplace les éléments du tableau[0..i-1] qui sont supérieurs à clé
            while (j >= 0 && this.chores[j].getDeadline().isAfter(cle.getDeadline())) {
                this.chores[j + 1] = this.chores[j];
                j = j - 1;
            }
            this.chores[j + 1] = cle;
        }
	}

	public void senioritySort() {
		int n = this.chores.length;
        for (int i = 1; i < n; i++) {
            Task cle = this.chores[i];
            int j = i - 1;

            // Déplace les éléments du tableau[0..i-1] qui sont supérieurs à clé
            while (j >= 0 && this.chores[j].getCreationDate().isAfter(cle.getCreationDate())) {
                this.chores[j + 1] = this.chores[j];
                j = j - 1;
            }
            this.chores[j + 1] = cle;
        }
	}

	public void durationSort() {
		int n = this.chores.length;
        for (int i = 1; i < n; i++) {
            Task cle = this.chores[i];
            int j = i - 1;

            // Déplace les éléments du tableau[0..i-1] qui sont supérieurs à clé
            while (j >= 0 && this.chores[j].getDuration() > cle.getDuration()) {
                this.chores[j + 1] = this.chores[j];
                j = j - 1;
            }
            this.chores[j + 1] = cle;
        }
	}

	public Task[] getTasks() {
		return this.chores.clone();
	}

	public Task getTask(int index) {
		return this.chores[index];
	}

}